require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

require 'savon/mock/spec_helper'

describe HeartlandPortico do
  include Savon::SpecHelper
  before(:all) { savon.mock!   }
  after(:all)  { savon.unmock! }

  let(:client) { HeartlandPortico.new(config['credentials']) }
  let(:config) {
    YAML.load(File.read('./config/spec.yml'))
  }

  describe '#credit_account_verify' do
    let(:card_data) {{
      :manual_entry => {
        :card_nbr  => '${#Project#CardNumberVISA}',
        :exp_month => '${#Project#ExpMonth}',
        :exp_year  => '${#Project#ExpYear}'
      }
    }}
    let(:card_holder_data) {{}}
    let(:params) {{
      :card_data => card_data,
      :card_holder_data => card_holder_data
    }}

    let(:expected_request) { xml_fixture("credit_account_verify_request_stub") }
    let(:success_response) { xml_fixture("credit_account_verify_response_success") }
    let(:auth_error_response) { xml_fixture("response_authentication_error") }

    describe "standard request" do
      let(:response) {
        savon.expects(:do_transaction).with(:message => expected_request).returns(success_response)
        client.credit_account_verify(params)
      }

      it { expect(response.message).to eq("CARD OK")}
      it { expect(response).to be_success }
    end

    describe "with bad authentication" do
      let(:response) {
        savon.expects(:do_transaction).with(:message => expected_request).returns(auth_error_response)
        client.credit_account_verify(params)
      }

      it { expect(response.message).to eq("ERROR")}
      it { expect(response).to_not be_success }
    end
  end
end

describe HeartlandPortico, :remote => true do
  let(:client) { HeartlandPortico.new(config['credentials']) }
  let(:config) {
    # Copy config/spec.yml to config/cert.yml and replace values
    # as provided by your certification contact to run remote specs.
    YAML.load(File.read('./config/cert.yml'))
  }

  describe '#credit_account_verify' do
    let(:card_data) {{ :manual_entry => config['test_visa'] }}
    let(:card_holder_data) {{}}
    let(:params) {{
      :card_data => card_data,
      :card_holder_data => card_holder_data
    }}

    it "returns a response" do
      response = client.credit_account_verify(params)
      expect(response).to be_success
      expect(response.message).to eq("CARD OK")
    end
  end


  describe '#credit_auth' do
    let(:card_data) {{ :manual_entry => config['test_visa'] }}
    let(:card_holder_data) {{}}
    let(:params) {{
      :allow_dup        => 'Y', # For tests only, obviously
      :amt              => amount,
      # gateway_txn_id    => Previous request transaction ID
      :card_data        => card_data,
      :card_holder_data => card_holder_data,
      :ecommerce        => "ECOM",
    }}
    let(:response) { client.credit_auth(params) }

    describe "for great success" do
      let(:amount) { "1.00" }

      it { expect(response).to be_success }
      it { expect(response).to be_successful_request }
      it { expect(response.message).to eq("APPROVAL") }
    end

    describe  "returns a decline response" do
      let(:amount) { "10.25" }

      it { expect(response).to_not be_success }
      it { expect(response).to be_successful_request }
      it { expect(response.message).to eq("DECLINE") }
    end
  end
end
